import _extends from 'babel-runtime/helpers/extends';
import _classCallCheck from 'babel-runtime/helpers/classCallCheck';
import uuid from 'uuid/v1';
import { mergeWith } from 'lodash';
import FieldsModel from './FieldsModel';
import FormsModel from './FormsModel';
import FormItemModelPublic from './public/FormItemModelPublic';

var FormItemModel = function FormItemModel(name, validator) {
  var _this = this;

  var excluded = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : false;

  _classCallCheck(this, FormItemModel);

  this.getPublic = function () {
    return _this['public'];
  };

  this.validate = function () {
    _this.fields.validate();
    _this.forms.validate();
  };

  this.reload = function () {
    _this.fields.reload();
    _this.forms.reload();
  };

  this.addErrors = function () {
    var parentErrors = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};

    var customizer = function customizer(objValue, srcValue) {
      if (Array.isArray(objValue)) {
        return objValue.concat(srcValue);
      }
    };
    var errors = _this.validator(_this, _this.fields, _this.fields.data());
    var mergedErrors = mergeWith(_extends({}, parentErrors), errors, customizer);
    _this.fields.addErrors(mergedErrors);
    _this.forms.addErrors(mergedErrors);
  };

  this.isValid = function () {
    return _this.fields.isValid() && _this.forms.isValid();
  };

  this.isTouched = function () {
    return _this.fields.isTouched() || _this.forms.isTouched();
  };

  this.init = function () {
    var data = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};

    _this.fields.init(data);
    _this.forms.init(data);
  };

  this.reset = function () {
    _this.fields.reset();
    _this.forms.reset();
  };

  this.makeDirty = function () {
    _this.fields.makeDirty();
    _this.forms.makeDirty();
  };

  this.makeSubmitted = function () {
    _this.fields.makeSubmitted();
    _this.forms.makeSubmitted();
  };

  this.data = function () {
    return _extends({}, _this.fields.data(), _this.forms.data());
  };

  this.values = function () {
    return _extends({}, _this.fields.values(), _this.forms.values());
  };

  this.getErrors = function () {
    return {
      fields: _this.fields.errors(),
      forms: _this.forms.errors()
    };
  };

  this.id = uuid();
  this.name = name;
  this.excluded = excluded;
  this.validator = validator || function () {
    return [];
  };
  this.fields = new FieldsModel();
  this.forms = new FormsModel();
  this['public'] = new FormItemModelPublic(this);
};

export { FormItemModel };